@@ -1,10 +1,10 @@
-import os
 import re
 import types
 
-from astvisitor import EmptyCode, descend, parse, ASTVisitor
+from astvisitor import EmptyCode, descend, parse_fragment, ASTVisitor
 from store import Class, Function, TestClass, TestMethod, ModuleNotFound, \
-     LiveObject, MethodCall, Method, Value, Type, Repr, Project, PointOfEntry
+     LiveObject, MethodCall, Method, Value, Type, Repr, Project, PointOfEntry, \
+     GeneratorObject, can_be_constructed
 from util import RePatternType, camelize, underscore, sorted, \
      regexp_flags_as_string, groupby
 
@@ -21,9 +21,13 @@ def unwrap_type(object):
     elif isinstance(object, Type):
         return object.type
     else:
-        raise ValueNeeded()
+        raise ValueNeeded("Can't unwrap the type of %s" % object)
 
-# :: ObjectWrapper -> string
+# :: [string] -> string
+def list_of(strings):
+    return "[%s]" % ', '.join(strings)
+
+# :: ObjectWrapper | [ObjectWrapper] -> string
 def type_as_string(object):
     """Return a most common representation of the wrapped object type.
 
@@ -33,12 +37,18 @@ def type_as_string(object):
     'dict'
     >>> type_as_string(Type(lambda: None))
     'types.FunctionType'
+    >>> type_as_string([Type(()), Type({})])
+    '[tuple, dict]'
     """
+    if isinstance(object, list):
+        return list_of(map(type_as_string, object))
+
     mapping = {
         list: 'list',
         dict: 'dict',
         tuple: 'tuple',
-        types.FunctionType: 'types.FunctionType'
+        types.FunctionType: 'types.FunctionType',
+        types.GeneratorType: 'types.GeneratorType'
     }
     try:
         return mapping[unwrap_type(object)]
@@ -65,12 +75,18 @@ class CallString(str):
 
     `imports` is a list of imports that this call requires.
     """
-    def __new__(cls, string, uncomplete=False, imports=[]):
+    def __new__(cls, string, uncomplete=False, imports=None):
+        if imports is None:
+            imports = set()
         call_string = str.__new__(cls, string)
         call_string.uncomplete = uncomplete
         call_string.imports = imports
         return call_string
 
+    def extend(self, value, uncomplete=False, imports=set()):
+        return CallString(value, self.uncomplete or uncomplete,
+                          self.imports.union(imports))
+
 # :: object -> CallString
 def standard_constructor_as_string(object):
     """For a given Python object return a string representing a code that will
@@ -91,15 +107,18 @@ def standard_constructor_as_string(object):
         # This may not always be right, but it's worth a try.
         return CallString(repr(object))
 
-# :: ObjectWrapper | LiveObject -> CallString
+# :: ObjectWrapper | LiveObject | list -> CallString
 def constructor_as_string(object):
-    """For a given object (either ObjectWrapper or a LiveObject instance) return
-    a string representing a code that will construct it.
+    """For a given object (either ObjectWrapper, a LiveObject instance or list
+    of any combination of those) return a string representing a code that will
+    construct it.
 
     >>> constructor_as_string(Value(123))
     '123'
     >>> constructor_as_string(Value('string'))
     "'string'"
+    >>> constructor_as_string([Value(1), Value('two')])
+    "[1, 'two']"
     >>> obj = LiveObject(None, Class('SomeClass'), PointOfEntry(Project('.'), 'poe'))
     >>> constructor_as_string(obj)
     'SomeClass()'
@@ -120,6 +139,8 @@ def constructor_as_string(object):
         return CallString("<TODO: %s>" % object.type.__name__, uncomplete=True)
     elif isinstance(object, Repr):
         return CallString("<TODO: %s>" % object.repr, uncomplete=True)
+    elif isinstance(object, list):
+        return list_of(map(constructor_as_string, object))
     else:
         raise TypeError("constructor_as_string expected ObjectWrapper or LiveObject object at input, not %s" % object)
 
@@ -162,6 +183,8 @@ def object2id(object):
             return 'true'
         elif object.value is False:
             return 'false'
+        elif isinstance(object.value, (list, dict, tuple)):
+            return underscore(type(object.value).__name__)
         elif isinstance(object.value, Exception):
             return underscore(exception_as_string(object))
         elif isinstance(object.value, RePatternType):
@@ -174,6 +197,14 @@ def object2id(object):
     else:
         raise TypeError("object2id() should be called with a ObjectWrapper argument, not %s" % object)
 
+def objects_list_to_id(objects):
+    """Convert given list of objects into string that can be used as an
+    identifier.
+    """
+    if not objects:
+        return 'nothing'
+    return '_then_'.join(map(object2id, objects))
+
 def input_as_string(input):
     """Generate an underscored description of given input arguments.
 
@@ -187,28 +218,28 @@ def input_as_string(input):
     return "_and_".join(["%s_equal_%s" % (arg, object2id(value))
                          for arg, value in sorted(input.iteritems())])
 
-def call2testname(object_name, input, output):
+def objcall2testname(object_name, input, output):
     """Generate a test method name that describes given object call.
 
-    >>> call2testname('do_this', {}, Value(True))
+    >>> objcall2testname('do_this', {}, Value(True))
     'test_do_this_returns_true'
-    >>> call2testname('compute', {}, Value('whatever you say'))
+    >>> objcall2testname('compute', {}, Value('whatever you say'))
     'test_compute_returns_whatever_you_say'
-    >>> call2testname('square', {'x': Value(7)}, Value(49))
+    >>> objcall2testname('square', {'x': Value(7)}, Value(49))
     'test_square_returns_49_for_7'
-    >>> call2testname('capitalize', {'str': Value('a word.')}, Value('A word.'))
+    >>> objcall2testname('capitalize', {'str': Value('a word.')}, Value('A word.'))
     'test_capitalize_returns_A_word_for_a_word'
 
     Two or more arguments are mentioned by name.
-        >>> call2testname('ackermann', {'m': Value(3), 'n': Value(2)}, Value(29))
+        >>> objcall2testname('ackermann', {'m': Value(3), 'n': Value(2)}, Value(29))
         'test_ackermann_returns_29_for_m_equal_3_and_n_equal_2'
 
     Will sort arguments alphabetically.
-        >>> call2testname('concat', {'s1': Value('Hello '), 's2': Value('world!')}, Value('Hello world!'))
+        >>> objcall2testname('concat', {'s1': Value('Hello '), 's2': Value('world!')}, Value('Hello world!'))
         'test_concat_returns_Hello_world_for_s1_equal_Hello_and_s2_equal_world'
 
     Always starts and ends a word with a letter or number.
-        >>> call2testname('strip', {'n': Value(1), 's': Value('  A bit of whitespace  ')}, Value(' A bit of whitespace '))
+        >>> objcall2testname('strip', {'n': Value(1), 's': Value('  A bit of whitespace  ')}, Value(' A bit of whitespace '))
         'test_strip_returns_A_bit_of_whitespace_for_n_equal_1_and_s_equal_A_bit_of_whitespace'
     """
     if input:
@@ -232,6 +263,33 @@ def exccall2testname(object_name, input, exception):
         call_description = object2id(exception)
     return "test_%s_raises_%s" % (underscore(object_name), call_description)
 
+def gencall2testname(object_name, input, yields):
+    """Generate a test method name that describes given generator object call
+    yielding some values.
+
+    >>> gencall2testname('generate', {}, [])
+    'test_generate_yields_nothing'
+    >>> gencall2testname('generate', {}, [Value(1), Value(2), Value(3)])
+    'test_generate_yields_1_then_2_then_3'
+    >>> gencall2testname('backwards', {'x': Value(321)}, [Value('one'), Value('two'), Value('three')])
+    'test_backwards_yields_one_then_two_then_three_for_321'
+    """
+    if input:
+        call_description = "%s_for_%s" % (objects_list_to_id(yields), input_as_string(input))
+    else:
+        call_description = objects_list_to_id(yields)
+    return "test_%s_yields_%s" % (underscore(object_name), call_description)
+
+def call2testname(call, object_name):
+    # Note: order is significant. We may have a GeneratorObject that raised
+    # an exception, and we care about exceptions more.
+    if call.raised_exception():
+        return exccall2testname(object_name, call.input, call.exception)
+    elif isinstance(call, GeneratorObject):
+        return gencall2testname(object_name, call.input, call.output)
+    else:
+        return objcall2testname(object_name, call.input, call.output)
+
 def sorted_test_method_descriptions(descriptions):
     return sorted(descriptions, key=lambda md: md.name)
 
@@ -243,12 +301,33 @@ def name2testname(name):
 def in_lambda(string):
     return "lambda: %s" % string
 
+def in_list(string):
+    return "list(%s)" % string
+
 def type_of(string):
     return "type(%s)" % string
 
+def map_types(string):
+    return "map(type, %s)" % string
+
+# :: (Call, CallString) -> CallString
+def decorate_call(call, string):
+    if isinstance(call, GeneratorObject):
+        invocations = len(call.output)
+        if call.raised_exception():
+            invocations += 1
+        # TODO: generators were added to Python 2.2, while itertools appeared in
+        # release  2.3, so we may generate incompatible tests here.
+        return string.extend("list(islice(%s, %d))" % (string, invocations),
+                             imports=[("itertools", "islice")])
+    return string
+
 def should_ignore_method(method):
     return method.name.startswith('_') and method.name != "__init__"
 
+def testable_calls(calls):
+    return [c for c in calls if c.is_testable()]
+
 class UnknownTemplate(Exception):
     def __init__(self, template):
         Exception.__init__(self, "Couldn't find template %r." % template)
@@ -358,7 +437,7 @@ class TestGenerator(object):
         # Don't generate empty test classes.
         if method_descriptions:
             test_body = self.create_test_class(class_name, method_descriptions)
-            test_code = parse(test_body)
+            test_code = parse_fragment(test_body)
             def methoddesc2testmethod(method_description):
                 name = method_description.name
                 return TestMethod(name=name, code=find_method_code(test_code, name))
@@ -374,9 +453,11 @@ class TestGenerator(object):
             return self._generate_test_method_descriptions_for_function(object, module)
         elif isinstance(object, Class):
             return self._generate_test_method_descriptions_for_class(object, module)
+        else:
+            raise TypeError("Don't know how to generate test method descriptions for %s" % object)
 
     def _generate_test_method_descriptions_for_function(self, function, module):
-        if function.calls:
+        if testable_calls(function.calls):
             # We're calling the function, so we have to make sure it will
             # be imported in the test
             self.ensure_import((module.locator, function.name))
@@ -409,19 +490,15 @@ class TestGenerator(object):
         return TestMethodDescription(name2testname(name))
 
     def _method_descriptions_from_function(self, function):
-        for call in function.get_unique_calls():
+        for call in testable_calls(function.get_unique_calls()):
+            name = call2testname(call, function.name)
             assertions = [self._create_assertion(function.name, call)]
 
-            if call.raised_exception():
-                name = exccall2testname(function.name, call.input, call.exception)
-            else:
-                name = call2testname(function.name, call.input, call.output)
-
             yield TestMethodDescription(name, assertions)
 
     def _method_description_from_live_object(self, live_object):
         init_call = live_object.get_init_call()
-        external_calls = live_object.get_external_calls()
+        external_calls = testable_calls(live_object.get_external_calls())
         local_name = underscore(live_object.klass.name)
         constructor = constructor_as_string(live_object)
         stub_all = constructor.uncomplete
@@ -434,15 +511,12 @@ class TestGenerator(object):
             else:
                 if len(external_calls) == 1:
                     call = external_calls[0]
-                    if call.raised_exception():
-                        test_name = exccall2testname(call.callable.name, call.input, call.exception)
-                    else:
-                        test_name = call2testname(call.callable.name, call.input, call.output)
+                    test_name = call2testname(call, call.definition.name)
                 # Methods with more than one external call use more brief
                 # descriptions that don't include inputs and outputs.
                 else:
                     methods = []
-                    for method, icalls in groupby(sorted([call.callable.name for call in external_calls])):
+                    for method, icalls in groupby(sorted([call.definition.name for call in external_calls])):
                         calls = list(icalls)
                         if len(calls) == 1:
                             methods.append(method)
@@ -460,9 +534,9 @@ class TestGenerator(object):
                     yield self._create_assertion(live_object.klass.name, init_call, stub_all)
                 else:
                     yield(('comment', "# Make sure it doesn't raise any exceptions."))
-    
+
             for call in external_calls:
-                name = "%s.%s" % (local_name, call.callable.name)
+                name = "%s.%s" % (local_name, call.definition.name)
                 yield(self._create_assertion(name, call, stub_all))
 
         def setup():
@@ -484,35 +558,40 @@ class TestGenerator(object):
         Generated assertion will be a stub if input of a call cannot be
         constructed or if stub argument is True.
         """
-        input = call_as_string(name, call.input)
+        callstring = decorate_call(call, call_as_string(name, call.input))
 
-        for import_ in input.imports:
+        for import_ in callstring.imports:
             self.ensure_import(import_)
 
         if call.raised_exception():
-            if input.uncomplete or stub:
+            if callstring.uncomplete or stub:
                 assertion_type = 'raises_stub'
             else:
                 assertion_type = 'raises'
             return (assertion_type,
                     exception_as_string(call.exception),
-                    in_lambda(input))
+                    in_lambda(callstring))
         else:
-            if input.uncomplete or stub:
+            if callstring.uncomplete or stub:
                 assertion_type = 'equal_stub'
             else:
                 assertion_type = 'equal'
 
-            if call.output.can_be_constructed:
-                return (assertion_type, constructor_as_string(call.output), input)
+            if can_be_constructed(call.output):
+                return (assertion_type, constructor_as_string(call.output),
+                        callstring)
             else:
                 # If we can't test for real values, let's at least test for the right type.
                 output_type = type_as_string(call.output)
+                if isinstance(call, GeneratorObject):
+                    callstring_type = map_types(callstring)
+                else:
+                    callstring_type = type_of(callstring)
                 self.ensure_import('types')
-                return (assertion_type, output_type, type_of(input))
+                return (assertion_type, output_type, callstring_type)
 
 class UnittestTestGenerator(TestGenerator):
-    main_snippet = parse("if __name__ == '__main__':\n    unittest.main()\n")
+    main_snippet = parse_fragment("if __name__ == '__main__':\n    unittest.main()\n")
 
     def test_class_header(self, name):
         self.ensure_import('unittest')
