@@ -8,14 +8,15 @@ import types
 from astvisitor import EmptyCode, Newline, create_import, find_last_leaf, \
      get_starting_whitespace, is_node_of_type, regenerate, \
      remove_trailing_whitespace
-from util import all_of_type, max_by_not_zero, set, \
+from util import all_of_type, max_by_not_zero, set, all, \
      write_string_to_file, ensure_directory, DirectoryException, \
-     get_last_modification_time, read_file_contents, python_modules_below, \
-     extract_subpath, directories_under, findfirst
+     get_last_modification_time, read_file_contents, is_generator_code, \
+     extract_subpath, directories_under, findfirst, contains_active_generator
 
 
-# So we can pickle the function type.
+# So we can pickle the function and generator types.
 __builtin__.function = types.FunctionType
+__builtin__.generator = types.GeneratorType
 
 class ModuleNeedsAnalysis(Exception):
     def __init__(self, path, out_of_sync=False):
@@ -47,13 +48,6 @@ def module_path_to_name(module_path):
 
 def module_path_to_test_path(module):
     """Convert a module locator to a proper test filename.
-
-    >>> module_path_to_test_path("module.py")
-    'test_module.py'
-    >>> module_path_to_test_path("pythoscope/store.py")
-    'test_pythoscope_store.py'
-    >>> module_path_to_test_path("pythoscope/__init__.py")
-    'test_pythoscope.py'
     """
     return "test_%s.py" % module_path_to_name(module)
 
@@ -136,14 +130,16 @@ class Project(object):
                 self.new_tests_directory = path
 
     def save(self):
+        # Try pickling the project first, because if this fails, we shouldn't
+        # save any changes at all.
+        pickled_project = pickle.dumps(self)
+
         # To avoid inconsistencies try to save all project's modules first. If
         # any of those saves fail, the pickle file won't get updated.
         for module in self.get_modules():
             module.save()
 
-        fd = open(self._get_pickle_path(), 'w')
-        pickle.dump(self, fd)
-        fd.close()
+        write_string_to_file(pickled_project, self._get_pickle_path())
 
     def find_module_by_full_path(self, path):
         subpath = self._extract_subpath(path)
@@ -154,7 +150,7 @@ class Project(object):
         if name not in self.points_of_entry:
             poe = PointOfEntry(project=self, name=name)
             self.points_of_entry[name] = poe
-            return poe
+        return self.points_of_entry[name]
 
     def remove_point_of_entry(self, name):
         poe = self.points_of_entry.pop(name)
@@ -199,7 +195,7 @@ class Project(object):
                 test_case.associated_modules.remove(old_module)
                 test_case.associated_modules.append(module)
             except ValueError:
-                pass        
+                pass
 
     def _extract_point_of_entry_subpath(self, path):
         """Takes the file path and returns subpath relative to the
@@ -338,23 +334,26 @@ class Project(object):
             for function in module.functions:
                 yield function
 
-    def find_class(self, name, modulepath):
+    def iter_generator_objects(self):
+        for module in self.iter_modules():
+            for generator in module.generators:
+                for gobject in generator.calls:
+                    yield gobject
+
+    def find_object(self, type, name, modulepath):
         modulename = self._extract_subpath(modulepath)
         try:
-            for klass in self[modulename].classes:
-                if klass.name == name:
-                    return klass
+            for obj in all_of_type(self[modulename].objects, type):
+                if obj.name == name:
+                    return obj
         except ModuleNotFound:
             pass
 
-    def find_function(self, name, modulepath):
-        modulename = self._extract_subpath(modulepath)
-        try:
-            for function in self[modulename].functions:
-                if function.name == name:
-                    return function
-        except ModuleNotFound:
-            pass
+# :: ObjectWrapper | [ObjectWrapper] -> bool
+def can_be_constructed(object):
+    if isinstance(object, list):
+        return all(map(can_be_constructed, object))
+    return object.can_be_constructed
 
 class ObjectWrapper(object):
     pass
@@ -372,6 +371,9 @@ class Value(ObjectWrapper):
         return isinstance(other, Value) and self.value == other.value
 
     def __hash__(self):
+        # Dictionary itself is not hashable, but set of its keys must be.
+        if isinstance(self.value, dict):
+            return hash(tuple(self.value.keys()))
         return hash(self.value)
 
     def __repr__(self):
@@ -415,7 +417,7 @@ class Repr(ObjectWrapper):
         return "Repr(%s)" % self.repr
 
 def is_pickable(object):
-    if isinstance(object, types.FunctionType):
+    if isinstance(object, (types.FunctionType, types.GeneratorType)):
         return False
     # TODO: handle more cases
     return True
@@ -447,13 +449,15 @@ class Call(object):
     __eq__ and __hash__ definitions provided for Function.get_unique_calls()
     and LiveObject.get_external_calls().
     """
-    def __init__(self, callable, input, output=None, exception=None):
+    def __init__(self, definition, input, output=None, exception=None):
         if [value for value in input.values() if not isinstance(value, ObjectWrapper)]:
             raise ValueError("All input values should be instances of ObjectWrapper class.")
         if output and exception:
             raise ValueError("Call should have a single point of return.")
+        if not isinstance(definition, Definition):
+            raise ValueError("Call definition object should be an instance of Definition.")
 
-        self.callable = callable
+        self.definition = definition
         self.input = input
         self.output = output
         self.exception = exception
@@ -479,21 +483,24 @@ class Call(object):
     def clear_exception(self):
         self.exception = None
 
+    def is_testable(self):
+        return True
+
     def __eq__(self, other):
-        return self.callable == other.callable and \
+        return self.definition == other.definition and \
                self.input == other.input and \
                self.output == other.output and \
                self.exception == other.exception
 
     def __hash__(self):
-        return hash((self.callable.name,
+        return hash((self.definition.name,
                      tuple(self.input.iteritems()),
                      self.output,
                      self.exception))
 
     def __repr__(self):
-        return "%s(callable=%r, input=%r, output=%r, exception=%r)" % \
-               (self.__class__.__name__, self.callable, self.input,
+        return "%s(definition=%s, input=%r, output=%r, exception=%r)" % \
+               (self.__class__.__name__, self.definition.name, self.input,
                 self.output, self.exception)
 
 class FunctionCall(Call):
@@ -505,11 +512,12 @@ class MethodCall(Call):
     pass
 
 class Definition(object):
-    def __init__(self, name, code=None):
+    def __init__(self, name, code=None, is_generator=False):
         if code is None:
             code = EmptyCode()
         self.name = name
         self.code = code
+        self.is_generator = is_generator
 
 class Callable(object):
     def __init__(self, calls=None):
@@ -520,9 +528,17 @@ class Callable(object):
     def add_call(self, call):
         self.calls.append(call)
 
+    def get_generator_object(self, unique_id):
+        def is_matching_gobject(call):
+            return isinstance(call, GeneratorObject) and call.unique_id == unique_id
+        return findfirst(is_matching_gobject, self.calls)
+
+    def remove_calls_from(self, point_of_entry):
+        self.calls = [call for call in self.calls if call.point_of_entry is not point_of_entry]
+
 class Function(Definition, Callable):
-    def __init__(self, name, code=None, calls=None):
-        Definition.__init__(self, name, code)
+    def __init__(self, name, code=None, calls=None, is_generator=False):
+        Definition.__init__(self, name, code, is_generator)
         Callable.__init__(self, calls)
 
     def is_testable(self):
@@ -531,9 +547,6 @@ class Function(Definition, Callable):
     def get_unique_calls(self):
         return set(self.calls)
 
-    def remove_calls_from(self, point_of_entry):
-        self.calls = [call for call in self.calls if call.point_of_entry is not point_of_entry]
-
     def __repr__(self):
         return "Function(name=%r, calls=%r)" % (self.name, self.calls)
 
@@ -543,6 +556,39 @@ class Function(Definition, Callable):
 class Method(Definition):
     pass
 
+class GeneratorObject(Call):
+    """Representation of a generator object - a callable with on input and many
+    outputs (here called "yields").
+
+    Although a generator object execution is not a single call, but consists of
+    a series of suspensions and resumes, we make it conform to the Call interface
+    for simplicity.
+    """
+    def __init__(self, id, generator, point_of_entry, input, yields=None, exception=None):
+        if yields is None:
+            yields = []
+        Call.__init__(self, generator, input, yields, exception)
+
+        self.id = id
+        self.point_of_entry = point_of_entry
+        self.unique_id = (point_of_entry.name, id)
+
+    def set_output(self, output):
+        self.output.append(wrap_object(output))
+
+    def is_testable(self):
+        return self.raised_exception() or self.output
+
+    def __hash__(self):
+        return hash((self.definition.name,
+                     tuple(self.input.iteritems()),
+                     tuple(self.output),
+                     self.exception))
+
+    def __repr__(self):
+        return "GeneratorObject(id=%d, generator=%r, yields=%r)" % \
+               (self.id, self.definition.name, self.output)
+
 class LiveObject(Callable):
     """Representation of an object which creation and usage was traced
     during dynamic inspection.
@@ -553,20 +599,24 @@ class LiveObject(Callable):
     identification purposes.
     """
     def __init__(self, id, klass, point_of_entry):
+        Callable.__init__(self)
+
         self.id = id
         self.klass = klass
         self.point_of_entry = point_of_entry
 
         self.unique_id = (point_of_entry.name, id)
-        self.calls = []
 
     def add_call(self, call):
-        self.calls.append(call)
+        # Don't add the same GeneratorObject more than once.
+        if isinstance(call, GeneratorObject) and call in self.calls:
+            return
+        Callable.add_call(self, call)
 
     def get_init_call(self):
         """Return a call to __init__ or None if it wasn't called.
         """
-        return findfirst(lambda call: call.callable.name == '__init__', self.calls)
+        return findfirst(lambda call: call.definition.name == '__init__', self.calls)
 
     def get_external_calls(self):
         """Return all calls to this object made from the outside.
@@ -574,7 +624,7 @@ class LiveObject(Callable):
         Note: __init__ is considered an internal call.
         """
         def is_not_init_call(call):
-            return call.callable.name != '__init__'
+            return call.definition.name != '__init__'
         def is_external_call(call):
             return (not call.caller) or (call.caller not in self.calls)
         return filter(is_not_init_call, filter(is_external_call, self.calls))
@@ -609,7 +659,7 @@ class Class(object):
         traced_method_names = set()
         for live_object in self.live_objects.values():
             for call in live_object.calls:
-                traced_method_names.add(call.callable.name)
+                traced_method_names.add(call.definition.name)
         return traced_method_names
 
     def get_untraced_methods(self):
@@ -779,7 +829,7 @@ class Localizable(object):
         self.created = created
 
     def _get_locator(self):
-        return re.sub(r'(%s__init__)?\.py$' % os.path.sep, '', self.subpath).\
+        return re.sub(r'(%s__init__)?\.py$' % re.escape(os.path.sep), '', self.subpath).\
             replace(os.path.sep, ".")
     locator = property(_get_locator)
 
@@ -926,13 +976,16 @@ class PointOfEntry(Localizable):
     """
     def __init__(self, project, name):
         poes_subpath = project._extract_subpath(project._get_points_of_entry_path())
-        Localizable.__init__(self, project, os.path.join(poes_subpath, name))
+        # Points of entry start with created attribute equal to 0, as they are
+        # not up-to-date until they're run. See finalize_inspection().
+        Localizable.__init__(self, project, os.path.join(poes_subpath, name), created=0)
 
         self.name = name
         # After an inspection run, this will be a reference to the top level call.
         self.call_graph = None
 
         self._preserved_objects = []
+        self._gobjects = []
 
     def get_path(self):
         return os.path.join(self.project._get_points_of_entry_path(), self.name)
@@ -947,41 +1000,97 @@ class PointOfEntry(Localizable):
             function.remove_calls_from(self)
         self.call_graph = None
 
-    def create_method_call(self, name, classname, modulepath, object, input):
-        klass = self.project.find_class(classname, modulepath)
+    def create_method_call(self, name, classname, modulepath, object, input, code, frame):
+        try:
+            live_object, method = self._retrieve_live_object_for_method(object, name, classname, modulepath)
+            if is_generator_code(code):
+                call = self._retrieve_generator_object(live_object, method, input, code, frame)
+            else:
+                call = MethodCall(method, wrap_call_arguments(input))
+            live_object.add_call(call)
+            return call
+        except LookupError:
+            pass
+
+    def create_function_call(self, name, modulepath, input, code, frame):
+        function = self.project.find_object(Function, name, modulepath)
+        if function:
+            if is_generator_code(code):
+                return self._retrieve_generator_object(function, function, input, code, frame)
+            else:
+                call = FunctionCall(self, function, wrap_call_arguments(input))
+                function.add_call(call)
+                return call
+
+    def finalize_inspection(self):
+        # We can release preserved objects now.
+        self._preserved_objects = []
+        # Mark the point of entry as up-to-date.
+        self.created = time.time()
+        # Fix output of generator objects.
+        self._fix_generator_objects()
+
+    def _retrieve_live_object_for_method(self, object, name, classname, modulepath):
+        klass = self.project.find_object(Class, classname, modulepath)
         if not klass:
-            return
+            raise LookupError("Couldn't find class %s in %s." %\
+                                  (classname, modulepath))
 
         method = klass.find_method_by_name(name)
         if not method:
-            return
+            raise LookupError("Couldn't find method %s in %s:%s." %\
+                                  (name, modulepath, classname))
 
-        call = MethodCall(method, wrap_call_arguments(input))
+        live_object = self._retrieve_live_object(object, klass)
 
+        return live_object, method
+
+    def _retrieve_generator_object(self, callable, generator, input, code, frame):
+        gobject = callable.get_generator_object((self.name, id(code)))
+
+        if not gobject:
+            gobject = GeneratorObject(id(code), generator, self, wrap_call_arguments(input))
+            callable.add_call(gobject)
+            self._gobjects.append(gobject)
+            self._preserve(code)
+
+            # Generator objects return None to the tracer when stopped. That
+            # extra None we have to filter out manually (see
+            # _fix_generator_objects method). The only way to distinguish
+            # between active and stopped generators is to ask garbage collector
+            # about them. So we temporarily save the generator frame inside the
+            # GeneratorObject, so it can be inspected later.
+            gobject._frame = frame
+
+        return gobject
+
+    def _retrieve_live_object(self, object, klass):
         try:
             live_object = klass.live_objects[(self.name, id(object))]
         except KeyError:
             live_object = LiveObject(id(object), klass, self)
             klass.add_live_object(live_object)
             self._preserve(object)
-
-        live_object.add_call(call)
-        return call
-
-    def create_function_call(self, name, modulepath, input):
-        function = self.project.find_function(name, modulepath)
-
-        if function:
-            call = FunctionCall(self, function, wrap_call_arguments(input))
-            function.add_call(call)
-            return call
-
-    def finalize_inspection(self):
-        # We can release preserved objects now.
-        self._preserved_objects = []
+        return live_object
 
     def _preserve(self, object):
         """Preserve an object from garbage collection, so its id won't get
         occupied by any other object.
         """
         self._preserved_objects.append(object)
+
+    def _fix_generator_objects(self):
+        """Remove last yielded values of generator objects, as those are
+        just bogus Nones placed on generator stop.
+        """
+        for gobject in self._gobjects:
+            if not contains_active_generator(gobject._frame) \
+                   and gobject.output \
+                   and gobject.output[-1] == Value(None):
+                gobject.output.pop()
+            # Once we know if the generator is active or not, we can discard
+            # the frame.
+            del gobject._frame
+
+        self._gobjects = []
+
