@@ -1,5 +1,8 @@
+import gc
 import os
 import re
+import types
+import warnings
 
 # Portability code.
 try:
@@ -15,6 +18,15 @@ except NameError:
         alist.sort()
         return alist
 
+try:
+    all = all
+except NameError:
+    def all(iterable):
+        for element in iterable:
+            if not element:
+                return False
+        return True
+
 try:
     from itertools import groupby
 except ImportError:
@@ -40,6 +52,12 @@ except ImportError:
                 self.currvalue = self.it.next() # Exit on StopIteration
                 self.currkey = self.keyfunc(self.currvalue)
 
+try:
+    from os.path import samefile
+except ImportError:
+    def samefile(file1, file2):
+        return os.path.realpath(file1) == os.path.realpath(file2)
+
 def camelize(name):
     """Covert name into CamelCase.
 
@@ -157,7 +175,9 @@ def ensure_directory(directory):
 
 def get_last_modification_time(path):
     try:
-        return os.path.getmtime(path)
+        # Casting to int, because we don't need better resolution anyway and it
+        # eases testing on different OSes.
+        return int(os.path.getmtime(path))
     except OSError:
         # File may not exist, in which case it was never modified.
         return 0
@@ -190,6 +210,21 @@ def findfirst(pred, seq):
         if pred(item):
             return item
 
+def contains_active_generator(frame):
+    return bool(all_of_type(gc.get_referrers(frame), types.GeneratorType))
+
+def is_generator_code(code):
+    return code.co_flags & 0x20 != 0
+
+def compile_without_warnings(stmt):
+    """Compile single interactive statement with Python interpreter warnings
+    disabled.
+    """
+    warnings.simplefilter('ignore')
+    code = compile(stmt, '', 'single')
+    warnings.resetwarnings()
+    return code
+
 # Regular expressions helpers.
 
 RePatternType = type(re.compile(''))
