@@ -1,15 +1,18 @@
+import difflib
 import os
 import re
+import types
 
 from fixture import TempIO
 from nose.tools import assert_equal
 
 from pythoscope.generator import add_tests_to_project
-from pythoscope.store import Module, Project, Function, ModuleNotFound, \
-     PointOfEntry
+from pythoscope.store import Function, ModuleNotFound, PointOfEntry, Project
 from pythoscope.util import read_file_contents, set
 
 
+UNPICKABLE_OBJECT = types.ClassType('class', (), {})
+
 DATA_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data'))
 
 def data(name):
@@ -19,6 +22,10 @@ def data(name):
 def read_data(name):
     return read_file_contents(data(name))
 
+def P(path):
+    "Convert given path with slashes to proper format for OS we're running on."
+    return os.path.join(*path.split("/"))
+
 def quoted_block(text):
     return ''.join(["> %s" % line for line in text.splitlines(True)])
 
@@ -52,12 +59,20 @@ def assert_single_function(info, name):
 def assert_equal_sets(collection1, collection2):
     assert_equal(set(collection1), set(collection2))
 
+def assert_equal_strings(s1, s2):
+    assert_equal(s1, s2, "Strings not equal. Diff:\n\n%s" % ''.join(difflib.ndiff(s1.splitlines(True), s2.splitlines(True))))
+
 def assert_not_raises(exception, callable):
     try:
         callable()
     except exception:
         assert False, "Exception %s has been raised." % exception
 
+def assert_instance(object, type):
+    assert isinstance(object, type), \
+           "Expected object %r to be of type %r, it was of type %r instead." % \
+           (object, type, type(object))
+
 class PointOfEntryMock(PointOfEntry):
     def __init__(self, project=None, name="poe", content=""):
         if project is None:
@@ -116,7 +131,7 @@ def get_test_module_contents(project):
     """Get contents of the first test module of a project.
     """
     try:
-        return project["tests/test_module.py"].get_content()
+        return project[P("tests/test_module.py")].get_content()
     except ModuleNotFound:
         return "" # No test module was generated.
 get_test_module_contents.__test__ = False
