@@ -2,16 +2,15 @@ import sys
 import types
 
 from nose.tools import assert_equal
-from nose.plugins.skip import SkipTest
 
 from pythoscope.inspector.dynamic import trace_function, trace_exec, \
      inspect_point_of_entry, setup_tracing, teardown_tracing
-from pythoscope.store import Class, Function, Method, LiveObject, \
-     Project, wrap_call_arguments, wrap_object, Type
+from pythoscope.store import Class, Function, GeneratorObject, \
+     Method, LiveObject, Project, wrap_call_arguments, wrap_object, Type
 from pythoscope.util import findfirst
 
 from helper import TestableProject, assert_length, PointOfEntryMock, \
-     assert_equal_sets, ProjectWithModules, ProjectInDirectory
+     assert_equal_sets, ProjectWithModules, ProjectInDirectory, assert_instance
 
 
 class ClassMock(Class):
@@ -28,8 +27,8 @@ class ClassMock(Class):
         return self._methods[name]
 
 class ProjectMock(Project):
-    """Project that has all the classes and functions you try to find inside it
-    via find_class() and find_function().
+    """Project that has all the classes, functions and generators you try to
+    find inside it via find_object().
     """
     def __init__(self, ignored_functions=[]):
         self.ignored_functions = ignored_functions
@@ -37,19 +36,16 @@ class ProjectMock(Project):
         self._classes = {}
         self._functions = {}
 
-    def find_class(self, classname, modulepath):
-        class_id = (classname, modulepath)
-        if not self._classes.has_key(class_id):
-            self._classes[class_id] = ClassMock(classname)
-        return self._classes[class_id]
-
-    def find_function(self, name, modulepath):
-        if name in self.ignored_functions:
+    def find_object(self, type, name, modulepath):
+        if type is Function and name in self.ignored_functions:
             return None
-        function_id = (name, modulepath)
-        if not self._functions.has_key(function_id):
-            self._functions[function_id] = Function(name)
-        return self._functions[function_id]
+
+        object_id = (name, modulepath)
+        container = self._get_container_for(type)
+
+        if not container.has_key(object_id):
+            container[object_id] = self._create_object(type, name)
+        return container[object_id]
 
     def iter_callables(self):
         for klass in self._classes.values():
@@ -61,6 +57,20 @@ class ProjectMock(Project):
     def get_callables(self):
         return list(self.iter_callables())
 
+    def _get_container_for(self, type):
+        if type is Class:
+            return self._classes
+        elif type is Function:
+            return self._functions
+        else:
+            raise TypeError("Cannot store %r inside a module." % type)
+
+    def _create_object(self, type, name):
+        if type is Class:
+            return ClassMock(name)
+        else:
+            return type(name)
+
 def assert_call(expected_input, expected_output, call):
     assert_equal(wrap_call_arguments(expected_input), call.input)
     assert not call.raised_exception()
@@ -71,9 +81,14 @@ def assert_call_with_exception(expected_input, expected_exception_type, call):
     assert call.raised_exception()
     assert_equal(expected_exception_type, type(call.exception.value))
 
+def assert_generator_object(expected_input, expected_yields, object):
+    assert_instance(object, GeneratorObject)
+    assert_equal(wrap_call_arguments(expected_input), object.input)
+    assert_equal(map(wrap_object, expected_yields), object.output)
+
 def call_graph_as_string(call_or_calls, indentation=0):
     def lines(call):
-        yield "%s%s()\n" % (" "*indentation, call.callable.name)
+        yield "%s%s()\n" % (" "*indentation, call.definition.name)
         for subcall in call.subcalls:
             yield call_graph_as_string(subcall, indentation+4)
 
@@ -288,6 +303,54 @@ def function_with_ignored_function():
         return ignored(z-1) * 3
     not_ignored_outer(13)
 
+def function_calling_generator():
+    def generator(x):
+        yield x
+        yield x + 1
+        yield x * 2
+        yield x ** 3
+    [x for x in generator(2)]
+
+def function_calling_generator_that_yields_one():
+    def generator():
+        yield 1
+    g = generator()
+    g.next()
+
+def function_calling_generator_that_yields_none():
+    def generator():
+        yield None
+    g = generator()
+    g.next()
+
+def function_calling_empty_generator():
+    def generator(x):
+        if False:
+            yield 'something'
+    [x for x in generator(123)]
+
+def function_calling_generator_that_doesnt_get_destroyed():
+    def generator():
+        yield 1
+    g = generator()
+    g.next()
+    globals()['__generator_yielding_one'] = g
+
+def function_calling_generator_that_yields_none_and_doesnt_get_destroyed():
+    def generator():
+        yield None
+    g = generator()
+    g.next()
+    globals()['__generator_yielding_none'] = g
+
+def function_calling_generator_method():
+    class Class(object):
+        def genmeth(self):
+            yield 0
+            yield 1
+            yield 0
+    [x for x in Class().genmeth()]
+
 class DynamicInspectorTest:
     def _collect_callables(self, fun, ignored_functions=[]):
         if isinstance(fun, str):
@@ -376,7 +439,7 @@ class TestTraceFunction(DynamicInspectorTest):
         assert_call({'x': 2, 'rest': (3,)},   [3,2],   function.calls[1])
         assert_call({'x': 4, 'rest': (5, 6)}, [5,6,4], function.calls[2])
 
-    def test_returns_function_objects_with_calls_that_use_varargs(self):
+    def test_returns_function_objects_with_calls_that_use_varargs_only(self):
         trace = self._collect_callables(function_calling_another_with_varargs_only)
         function = trace.pop()
 
@@ -433,13 +496,13 @@ class TestTraceFunction(DynamicInspectorTest):
         assert_length(trace, 2)
         assert all(isinstance(obj, LiveObject) for obj in trace)
         assert_equal_sets(['strange_method', 'another_strange_method'],
-                          [obj.calls[0].callable.name for obj in trace])
+                          [obj.calls[0].definition.name for obj in trace])
 
     def test_distinguishes_between_methods_with_the_same_name_from_different_classes(self):
         trace = self._collect_callables(function_calling_two_methods_with_the_same_name_from_different_classes)
 
         assert_equal_sets([('FirstClass', 1, 'method'), ('SecondClass', 1, 'method')],
-                          [(obj.klass.name, len(obj.calls), obj.calls[0].callable.name) for obj in trace])
+                          [(obj.klass.name, len(obj.calls), obj.calls[0].definition.name) for obj in trace])
 
     def test_distinguishes_between_classes_and_functions(self):
         trace = self._collect_callables(function_calling_other_which_uses_name_and_module_variables)
@@ -452,16 +515,16 @@ class TestTraceFunction(DynamicInspectorTest):
         assert_length(trace, 1)
 
         live_object = trace[0]
-        assert isinstance(live_object, LiveObject)
+        assert_instance(live_object, LiveObject)
         assert_length(live_object.calls, 2)
         assert_length(live_object.get_external_calls(), 1)
 
         external_call = live_object.get_external_calls()[0]
-        assert_equal('method', external_call.callable.name)
+        assert_equal('method', external_call.definition.name)
         assert_length(external_call.subcalls, 1)
 
         subcall = external_call.subcalls[0]
-        assert_equal('other_method', subcall.callable.name)
+        assert_equal('other_method', subcall.definition.name)
 
     def test_creates_a_call_graph_of_execution_for_nested_calls(self):
         self._collect_callables(function_with_nested_calls)
@@ -512,6 +575,72 @@ class TestTraceFunction(DynamicInspectorTest):
         assert_call({'z': 13}, 75, outer_function.calls[0])
         assert_call({'x': 24}, 25, inner_function.calls[0])
 
+    def test_handles_yielded_values(self):
+        trace = self._collect_callables(function_calling_generator)
+
+        assert_length(trace, 1)
+        generator = trace.pop()
+
+        assert_instance(generator, Function)
+        assert_length(generator.calls, 1)
+        gobject = generator.calls.pop()
+
+        assert_generator_object({'x': 2}, [2, 3, 4, 8], gobject)
+
+    def test_handles_single_yielded_value(self):
+        trace = self._collect_callables(function_calling_generator_that_yields_one)
+
+        assert_length(trace, 1)
+        generator = trace.pop()
+
+        assert_instance(generator, Function)
+        assert_length(generator.calls, 1)
+        gobject = generator.calls.pop()
+
+        assert_generator_object({}, [1], gobject)
+        assert not gobject.raised_exception()
+
+    def test_handles_yielded_nones(self):
+        trace = self._collect_callables(function_calling_generator_that_yields_none)
+
+        gobject = trace.pop().calls.pop()
+
+        assert_generator_object({}, [None], gobject)
+
+    def test_handles_empty_generators(self):
+        trace = self._collect_callables(function_calling_empty_generator)
+
+        gobject = trace.pop().calls.pop()
+
+        assert_generator_object({'x': 123}, [], gobject)
+
+    def test_handles_generator_objects_that_werent_destroyed(self):
+        trace = self._collect_callables(function_calling_generator_that_doesnt_get_destroyed)
+
+        gobject = trace.pop().calls.pop()
+
+        assert_generator_object({}, [1], gobject)
+
+    def test_handles_generator_objects_that_yield_none_and_dont_get_destroyed(self):
+        trace = self._collect_callables(function_calling_generator_that_yields_none_and_doesnt_get_destroyed)
+
+        gobject = trace.pop().calls.pop()
+
+        assert_generator_object({}, [None], gobject)
+
+    def test_handles_generator_methods(self):
+        trace = self._collect_callables(function_calling_generator_method)
+
+        assert_length(trace, 1)
+
+        live_object = trace.pop()
+        assert_instance(live_object, LiveObject)
+        assert_length(live_object.calls, 1)
+
+        gobject = live_object.calls[0]
+        assert_instance(gobject, GeneratorObject)
+        assert_generator_object({}, [0, 1, 0], gobject)
+
 class TestTraceExec(DynamicInspectorTest):
     "trace_exec"
     def test_returns_function_objects_with_all_calls_recorded(self):
