@@ -2,19 +2,18 @@ import os
 import re
 import time
 
-from fixture import TempIO
 from nose.tools import assert_equal, assert_not_equal, assert_raises
 
 from pythoscope.astvisitor import parse
 from pythoscope.generator import add_tests_to_project
-from pythoscope.store import Project, Module, Class, Method, Function, \
+from pythoscope.store import Project, Class, Method, Function, \
      ModuleNeedsAnalysis, ModuleSaveError, TestClass, TestMethod, \
      MethodCall, FunctionCall, LiveObject, wrap_call_arguments, \
-     wrap_object, PointOfEntry
+     wrap_object, PointOfEntry, GeneratorObject
 from pythoscope.util import read_file_contents, get_last_modification_time
 
 from helper import assert_contains, assert_doesnt_contain, assert_length,\
-     CustomSeparator, generate_single_test_module, ProjectInDirectory, \
+     generate_single_test_module, ProjectInDirectory, \
      ProjectWithModules, TestableProject, assert_contains_once, \
      PointOfEntryMock, get_test_cases, assert_equal_sets
 
@@ -23,22 +22,30 @@ add_tests_to_project.__test__ = False
 TestClass.__test__ = False
 TestMethod.__test__ = False
 
+def create_method_call(method, input, output, call_type, poe):
+    if call_type == 'output':
+        return MethodCall(method, wrap_call_arguments(input), output=wrap_object(output))
+    elif call_type == 'exception':
+        return MethodCall(method, wrap_call_arguments(input), exception=wrap_object(output()))
+    elif call_type == 'generator':
+        return GeneratorObject(12345, method, poe, wrap_call_arguments(input),
+                               map(wrap_object, output))
 
-def ClassWithMethods(classname, methods, exit_point='output'):
+def ClassWithMethods(classname, methods, call_type='output'):
+    """call_type has to be one of 'output', 'exception' or 'generator'.
+    """
     method_objects = []
     method_calls = []
+    poe = PointOfEntry(Project('.'), 'poe')
 
     for name, calls in methods:
         method = Method(name)
         method_objects.append(method)
         for input, output in calls:
-            if exit_point == 'output':
-                method_calls.append(MethodCall(method, wrap_call_arguments(input), output=wrap_object(output)))
-            elif exit_point == 'exception':
-                method_calls.append(MethodCall(method, wrap_call_arguments(input), exception=wrap_object(output())))
+            method_calls.append(create_method_call(method, input, output, call_type, poe))
 
     klass = Class(classname, methods=method_objects)
-    live_object = LiveObject(12345, klass, PointOfEntry(Project('.'), 'poe'))
+    live_object = LiveObject(12345, klass, poe)
     live_object.calls = method_calls
     klass.add_live_object(live_object)
 
@@ -62,6 +69,20 @@ def FunctionWithExceptions(funcname, calls):
 def FunctionWithSingleException(funcname, input, exception):
     return FunctionWithExceptions(funcname, [(input, exception)])
 
+def GeneratorWithYields(genname, input, yields):
+    poe = PointOfEntryMock()
+    generator = Function(genname, is_generator=True)
+    gobject = GeneratorObject(12345, generator, poe, wrap_call_arguments(input), map(wrap_object, yields))
+    generator.calls = [gobject]
+    return generator
+
+def GeneratorWithSingleException(genname, input, exception):
+    poe = PointOfEntryMock()
+    generator = Function(genname, is_generator=True)
+    gobject = GeneratorObject(12345, generator, poe, wrap_call_arguments(input), exception=wrap_object(exception()))
+    generator.calls = [gobject]
+    return generator
+
 class TestGenerator:
     def test_generates_unittest_boilerplate(self):
         result = generate_single_test_module(objects=[Function('function')])
@@ -356,6 +377,27 @@ class TestGenerator:
         assert_contains(result, "def test_higher_returns_true_for_function(self):")
         assert_contains(result, "# self.assertEqual(True, higher(f=<TODO: function>))")
 
+    def test_generates_assert_equal_type_for_functions_returning_generator_objects(self):
+        def generator():
+            yield 1
+        objects = [FunctionWithSingleCall('gengen', {}, generator())]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "import types")
+        assert_contains(result, "def test_gengen_returns_generator(self):")
+        assert_contains(result, "self.assertEqual(types.GeneratorType, type(gengen()))")
+
+    def test_generates_assert_equal_test_stub_for_functions_which_take_generator_objects_as_arguments(self):
+        def generator():
+            yield 1
+        objects = [FunctionWithSingleCall('ungen', {'g': generator()}, True)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_ungen_returns_true_for_generator(self):")
+        assert_contains(result, "# self.assertEqual(True, ungen(g=<TODO: generator>))")
+
     def test_generates_assert_equal_type_test_stub_for_functions_which_take_and_return_functions(self):
         objects = [FunctionWithSingleCall('highest', {'f': lambda: 42}, lambda: 43)]
 
@@ -399,6 +441,65 @@ class TestGenerator:
 
         assert_contains(result, "def test_look_at_2_times_and_modify(self):")
 
+    def test_generates_assert_equal_for_generator_functions(self):
+        objects = [GeneratorWithYields('random', {'seed': 1}, [1, 8, 7, 2])]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_random_yields_1_then_8_then_7_then_2_for_1(self):")
+        assert_contains(result, "self.assertEqual([1, 8, 7, 2], list(islice(random(seed=1), 4)))")
+
+    def test_generates_assert_equal_for_generator_methods(self):
+        klass = ClassWithMethods('SuperGenerator', [('degenerate', [({'what': 'strings'}, ['one', 'two'])])], call_type='generator')
+
+        result = generate_single_test_module(objects=[klass])
+
+        assert_contains(result, "def test_degenerate_yields_one_then_two_for_strings(self):")
+        assert_contains(result, "super_generator = SuperGenerator()")
+        assert_contains(result, "self.assertEqual(['one', 'two'], list(islice(super_generator.degenerate(what='strings'), 2)))")
+
+    def test_generates_assert_equal_stub_for_generator_functions_with_unpickable_inputs(self):
+        objects = [GeneratorWithYields('call_twice', {'x': lambda: 1}, [1, 1])]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_call_twice_yields_1_then_1_for_function(self):")
+        assert_contains(result, "# self.assertEqual([1, 1], list(islice(call_twice(x=<TODO: function>), 2)))")
+
+    def test_generates_assert_equal_types_for_generator_functions_with_unpickable_outputs(self):
+        objects = [GeneratorWithYields('lambdify', {'x': 1}, [lambda: 1, lambda: 2])]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_lambdify_yields_function_then_function_for_1(self):")
+        assert_contains(result, "self.assertEqual([types.FunctionType, types.FunctionType], map(type, list(islice(lambdify(x=1), 2))))")
+
+    def test_generates_assert_raises_for_generator_functions_with_exceptions(self):
+        objects = [GeneratorWithSingleException('throw', {'string': {}}, TypeError)]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "def test_throw_raises_type_error_for_dict(self):")
+        assert_contains(result, "self.assertRaises(TypeError, lambda: list(islice(throw(string={}), 1)))")
+
+    def test_takes_slice_of_generated_values_list_to_work_around_infinite_generators(self):
+        objects = [GeneratorWithYields('nats', {'start': 1}, [1, 2, 3])]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "from itertools import islice")
+        assert_contains(result, "def test_nats_yields_1_then_2_then_3_for_1(self):")
+        assert_contains(result, "self.assertEqual([1, 2, 3], list(islice(nats(start=1), 3)))")
+
+    def test_doesnt_test_unused_generators(self):
+        objects = [GeneratorWithYields('useless', {'anything': 123}, [])]
+
+        result = generate_single_test_module(objects=objects)
+
+        assert_contains(result, "assert False")
+        assert_doesnt_contain(result, "assertEqual")
+        assert_doesnt_contain(result, "assertRaises")
+
 class TestGeneratorWithTestDirectoryAsFile:
     def setUp(self):
         self.project = TestableProject()
