@@ -1,15 +1,20 @@
 import os
+from pickle import PicklingError
 
 from fixture import TempIO
 from nose.tools import assert_equal, assert_raises
 
-from pythoscope.store import Project, Module, Class, Function, TestClass, \
+from pythoscope.astvisitor import parse_fragment
+from pythoscope.generator import find_method_code
+from pythoscope.store import Project, Class, Function, TestClass, \
      TestMethod, ModuleNotFound
 from pythoscope.inspector import remove_deleted_modules
+from pythoscope.util import read_file_contents
 
 from helper import assert_length, assert_equal_sets, EmptyProject, \
      ProjectWithModules, ProjectWithRealModules, ProjectInDirectory, \
-     assert_not_raises, get_test_cases
+     assert_not_raises, get_test_cases, assert_equal_strings, \
+     UNPICKABLE_OBJECT, P
 
 # Let nose know that those aren't test classes.
 TestClass.__test__ = False
@@ -41,7 +46,7 @@ class TestProject:
         assert_equal(["Syntax error"], project['bad_module'].errors)
 
     def test_can_be_queried_for_modules_by_their_path(self):
-        paths = ["module.py", "sub/dir/module.py", "package/__init__.py"]
+        paths = ["module.py", P("sub/dir/module.py"), P("package/__init__.py")]
         project = ProjectWithModules(paths)
 
         for path in paths:
@@ -52,7 +57,7 @@ class TestProject:
         assert_raises(ModuleNotFound, lambda: project["whatever"])
 
     def test_can_be_queried_for_modules_by_their_locator(self):
-        paths = ["module.py", "sub/dir/module.py", "package/__init__.py"]
+        paths = ["module.py", P("sub/dir/module.py"), P("package/__init__.py")]
         locators = ["module", "sub.dir.module", "package"]
         project = ProjectWithModules(paths)
 
@@ -60,28 +65,27 @@ class TestProject:
             assert_equal(path, project[locator].subpath)
 
     def test_replaces_old_module_objects_with_new_ones_during_create_module(self):
-        paths = ["module.py", "sub/dir/module.py", "other/module.py"]
+        paths = ["module.py", P("sub/dir/module.py"), P("other/module.py")]
         project = ProjectWithModules(paths)
 
-        new_module = project.create_module("other/module.py")
+        new_module = project.create_module(P("other/module.py"))
 
         assert_length(project.get_modules(), 3)
-        assert project["other/module.py"] is new_module
+        assert project[P("other/module.py")] is new_module
 
     def test_replaces_module_instance_in_test_cases_associated_modules_during_module_replacement(self):
-        paths = ["module.py", "sub/dir/module.py", "other/module.py"]
+        paths = ["module.py", P("sub/dir/module.py"), P("other/module.py")]
         project = ProjectWithModules(paths)
-        test_class = TestClass(name='TestAnything', associated_modules=[project["other/module.py"]])
+        test_class = TestClass(name='TestAnything', associated_modules=[project[P("other/module.py")]])
         project.add_test_case(test_class)
 
-        new_module = project.create_module("other/module.py")
+        new_module = project.create_module(P("other/module.py"))
 
         assert_length(test_class.associated_modules, 1)
         assert test_class.associated_modules[0] is new_module
 
     def test_adds_new_test_methods_to_existing_test_classes_inside_application_modules(self):
         project = EmptyProject()
-        application_class = Class("Something", [])
         test_class = TestClass("TestSomething")
         module = project.create_module("somethings.py")
         module.add_test_case(test_class)
@@ -144,6 +148,22 @@ class TestProject:
         assert_raises(ModuleNotFound, lambda: project["other_module"])
         assert_not_raises(ModuleNotFound, lambda: project["test_module"])
 
+    def test_doesnt_save_uncomplete_pickle_files(self):
+        project = ProjectInDirectory()
+        project.save()
+        original_pickle = read_file_contents(project._get_pickle_path())
+
+        # Inject unpickable object into project.
+        project._injected_attr = UNPICKABLE_OBJECT
+        try:
+            project.save()
+        except PicklingError:
+            pass
+
+        # Make sure that the original file wasn't overwritten.
+        assert_equal_strings(original_pickle,
+                             read_file_contents(project._get_pickle_path()))
+
 class TestProjectWithTestModule:
     def setUp(self):
         self.project = EmptyProject()
@@ -151,12 +171,15 @@ class TestProjectWithTestModule:
         self.test_module = self.project.create_module("test_module.py")
         self.test_module.add_test_case(self.existing_test_class)
 
-    def test_attaches_test_class_to_test_module_with_most_test_cases_for_associated_module(self):
-        module = self.project.create_module("module.py")
-        irrelevant_test_module = self.project.create_module("irrelevant_test_module.py")
-        self.existing_test_class.associated_modules = [module]
+    def _associate_module_with_existing_test_class(self):
+        self.associated_module = self.project.create_module("module.py")
+        self.existing_test_class.associated_modules = [self.associated_module]
 
-        new_test_class = TestClass("new", associated_modules=[module])
+    def test_attaches_test_class_to_test_module_with_most_test_cases_for_associated_module(self):
+        self.project.create_module("irrelevant_test_module.py")
+        self._associate_module_with_existing_test_class()
+
+        new_test_class = TestClass("new", associated_modules=[self.associated_module])
         self.project.add_test_case(new_test_class)
 
         assert new_test_class in self.test_module.test_cases
@@ -230,3 +253,41 @@ class TestProjectWithTestModule:
         # But the method got replaced.
         assert_equal([new_test_method],
                      get_test_cases(self.project)[0].test_cases)
+
+    def test_appends_new_test_methods_to_test_classes_with_proper_indentation(self):
+        self._associate_module_with_existing_test_class()
+
+        klass = self._test_class_from_code(
+            "class NewTestClass(unittest.TestCase):\n"\
+            "    def test_some_method(self):\n"\
+            "        assert False # c'mon, implement me\n",
+            "NewTestClass",
+            ["test_some_method"],
+            [self.associated_module])
+        another_klass = self._test_class_from_code(
+            "class NewTestClass(unittest.TestCase):\n"\
+            "    def test_new_method(self):\n"\
+            "        assert True # ha!\n",
+            "NewTestClass",
+            ["test_new_method"],
+            [self.associated_module])
+        expected_output = "class NewTestClass(unittest.TestCase):\n"\
+                          "    def test_some_method(self):\n"\
+                          "        assert False # c'mon, implement me\n\n"\
+                          "    def test_new_method(self):\n"\
+                          "        assert True # ha!\n"
+
+        self.project.add_test_case(klass)
+        self.project.add_test_case(another_klass)
+
+        assert_equal_strings(expected_output, self.test_module.get_content())
+
+    def _test_class_from_code(self, code, name, method_names, associated_modules):
+        # TODO: this may too easily get out of sync with the code in
+        # generator:TestGenerator._generate_test_case, so refactor common things out.
+        parsed_code = parse_fragment(code)
+        def name2method(name):
+            return TestMethod(name=name, code=find_method_code(parsed_code, name))
+        return TestClass(name=name, code=parsed_code,
+                         test_cases=map(name2method, method_names),
+                         associated_modules=associated_modules)
